package hamsam.protocol.insecureSample;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
//import java.io.ObjectInputStream;
//import java.io.ObjectOutputStream;
//import java.io.IOException;
import java.io.*;
import java.net.Socket;
import java.util.Arrays;
import java.util.Hashtable;

import hamsam.api.Buddy;
import hamsam.api.IMListener;
import hamsam.api.Message;
import hamsam.api.Conference;
import hamsam.api.Response;
import hamsam.exception.UnsupportedOperationException;
import hamsam.net.ProxyInfo;
import hamsam.protocol.Protocol;

/**
 * This class implements the insecure protocol's wire format.
 * InsecureProtocolClient calls sendMessageToServer(), which serializes
 * client requests and sends them to the server.  receiveMessageFromServer 
 * unpacks these messages, and makes appropriate calls to InsecureServerLogic.
 * 
 * TODO RCS:  I made the arbitrary decision to pass in String objects when the recipient is a Buddy, 
 * and Conference objects when the recipient is a Conference.  You'll need to change 
 * InsecureProtocol.java if you decide to use a different convention.
 * 
 * @author sears
 *
 */
public class InsecureNetworking {

	// Enumerate constants here:  (Note: Java 1.4 doesn't support enumerations, and 1.4's switch{} only works with ints.)

	// These messages are processed server side and implement methods in InsecureServerLogic
	// There is a 1 to 1 correspondance between these constants and Protocol methods.

	public static final int CHANGE_STATUS           = 1;
	public static final int CONNECT                 = 2;
	public static final int DISCONNECT              = 3;
	public static final int START_CONFERENCE        = 4;
	public static final int QUIT_CONFERENCE         = 5;
	public static final int SEND_CONFERENCE_MESSAGE = 6;
	public static final int ADD_TO_BUDDY_LIST       = 7;
	public static final int DELETE_FROM_BUDDY_LIST  = 8;
	public static final int SEND_INSTANT_MESSAGE    = 9;

	// These are protocol specific.
	
	public static final int ADD_BUDDY_RESPONSE      = 10;
	public static final int CONFERENCE_INVITATION_RESPONSE = 11;
	
	// Add new message types directly above this line, and update the number below.  Make sure that it is still <= END_OF_LISTENER_MESSAGE_TYPE_LIST
	public static final int END_OF_PROTOCOL_MESSAGE_TYPE_LIST = 12;
	
	//..
	
	// These messages are processed client side and invoke IMListener implementations.
	
	public static final int CONNECTING                     = 255;
	public static final int CONNECTED                      = 254;
	public static final int CONNECT_FAILED                 = 253;
	public static final int DISCONNECTED                   = 252;
	public static final int BUDDY_ADD_REQUEST              = 251;
	public static final int BUDDY_ADDED                    = 250;
	public static final int BUDDY_ADD_REJECTED             = 249;
	public static final int BUDDY_ADD_FAILED               = 248;
	public static final int BUDDY_DELETED                  = 247;
	public static final int BUDDY_DELETE_FAILED            = 246;
	public static final int BUDDY_IGNORED                  = 245;
	public static final int BUDDY_IGNORE_FAILED            = 244;
	public static final int BUDDY_UNIGNORED                = 243;
	public static final int BUDDY_UNIGNORE_FAILED          = 242;
	public static final int BUDDY_LIST_RECEIVED            = 241;
	public static final int IGNORE_LIST_RECEIVED           = 240;
	public static final int BUDDY_STATUS_CHANGED           = 239;
	public static final int INSTANT_MESSAGE_RECEIVED       = 238;
	public static final int OFFLINE_MESSAGE_RECEIVED       = 237;
	public static final int PROTOCOL_MESAGE_RECEIVED       = 236;
	public static final int TYPING_STARTED                 = 235;
	public static final int TYPING_STOPPED                 = 234;
	public static final int CONFERENCE_INVITATION_RECEIVED = 233;
	public static final int CONFERENCE_INVITATION_DECLINED = 232;
	public static final int CONFERENCE_INVITATION_ACCEPTED = 231;
	public static final int CONFERENCE_MESSAGE_RECEIVED    = 230;
	public static final int CONFERENCE_PARTICIPANT_JOINED  = 229;
	public static final int CONFERENCE_PARTICIPANT_LEFT    = 228;
	public static final int CONFERENCE_CLOSED              = 227;
	public static final int MAIL_NOTIFICATION_RECEIVED     = 226;
	
	// TODO RCS I don't think all of these can be invoked by the server, so I stopped defining constants here.
	// Add new message types directly above this line, and udpate the number below.  Make sure that it is still >= END_OF_PROTOCOL_MESSAGE_TYPE_LIST
	public static final int END_OF_LISTENER_MESSAGE_TYPE_LIST = 225;
	
	// ------------------------------------------------------------------------------------------------------------------

    private ObjectOutputStream out = null;
    private ObjectInputStream in = null;
    private InsecureServerLogic ipsObject = null;	// InsecureServerLogic object
    private InsecureProtocol protocol = null;
    private String username = null;
    /**
     * This is used by the client side to map conference messages to persistant conference objects.
     * 
     * TODO it is kind of a kludge... but do we need to also do this with buddies?
     */ 
    public InsecureNetworking(ObjectInputStream incoming, ObjectOutputStream outgoing) {
	// This is for the server
		in = incoming;
		out = outgoing;
    }
    
    public void close()  { 
    	try { 
    		in.close();
    	} catch (IOException e) { 
    		e.printStackTrace();
    	} finally { 
    		try { 
    			out.close();
    		} catch (IOException e) { 
    			e.printStackTrace();
    		}
    	}
    }
	
	//	 TODO RCS Pick an appropriate interface for this constructor.
	public InsecureNetworking(String username, String password, String server, ProxyInfo info, InsecureProtocol protocol) {
        try { 
        	this.protocol = protocol;
        	System.out.println("Opening connection to " + server + " port 1234");
        	Socket socket = new Socket(server, 1234);
        	out = new ObjectOutputStream(socket.getOutputStream());
        	in = new ObjectInputStream(socket.getInputStream());
        } catch(Exception e) {
        	try {
        		System.out.println("Tearing down connections.");
	        	in.close();
	        	out.close();
        	} catch (IOException ioe) {
        		System.out.println("Error tearing down connections!");
        	}
        	e.printStackTrace();
        	System.err.println("Error creating InsecureNetworking object object.");
        	System.exit(-1);	// fail miserably
        }
	}

	protected void sendMessage(int type, Serializable[] payload) throws IOException {
		out.writeInt(type);
		out.writeObject(payload);
		out.flush();
	}
	/**
	 * @deprecated
	 */
	protected void sendMessage(Serializable subject, int type, Serializable payload) throws IOException { 
		out.writeInt(type);			// message type
		out.writeObject(subject);	// message recipient
		out.writeObject(payload);	// message payload
		out.flush();
	}

	/**
	 * I run client side.
	 * @param type
	 * @param payload
	 */
	public void sendMessageToServer(int type /* message type */, Serializable[] payload) { 
		if(type >= END_OF_PROTOCOL_MESSAGE_TYPE_LIST) {
			throw new IllegalStateException("Invalid message type");
		}
		try { 
			sendMessage(type, payload);
		} catch (IOException e) { 
			try {	// try to take down the incoming connection
				in.close();
			} catch (IOException ioe) {
				ioe.printStackTrace();
			}
			try {	// try to take down the outbound connection
				out.close();
			} catch (IOException ioe) {
				ioe.printStackTrace();
			}
			e.printStackTrace();
		}
	}
//	private static InsecureProtocol protocol = new InsecureProtocol();
	/**
	 * I run server side.
	 * TODO receiveMessageFromClient's interface needs to be defined.
	 */
	public boolean receiveMessageFromClient() {
	    //		receiveMessage();
		// step one:  Extract message id
	    // Defining Wirespec: First we have the messageid: int,
	    //                    Second we have the Buddy
	    //                    Third we have the Message
		
	    boolean returnCode = true;
		try {

			
		int messageType = in.readInt();
		Object[] messageObjects = (Object[])in.readObject();
		
		if(messageType >= END_OF_PROTOCOL_MESSAGE_TYPE_LIST) { 
			System.out.println("Bad message type from user;");
			return false;
		}
		
		// what to do with the socket now that we know the messagetype
		switch (messageType) {
			case CHANGE_STATUS :
				// make a call into InsecureServerLogic.
				this.ipsObject.changeStatus( (String)messageObjects[0] );
				break;
			case CONNECT :
				// on CONNECT (aka login)
				System.out.println("CONNECT received");
				username = (String)messageObjects[0];
				String password = (String)messageObjects[1];
				this.ipsObject = new InsecureServerLogic(username);
				returnCode = ipsObject.connect(this, username, password);
				System.out.println("Connect processed.");
				break;		
			case DISCONNECT :
				System.out.println("DISCONNECT received");
				this.ipsObject.disconnect();
				returnCode = false;	// make InsecureProtocolThread break out of its loop
				break;
			case START_CONFERENCE :
				this.ipsObject.startConference( (String)messageObjects[0],
												(String)messageObjects[1], 
												(String[])messageObjects[2]);
				break;			
			case QUIT_CONFERENCE :
				this.ipsObject.quitConference( (String)messageObjects[0] );
				break;			
			case SEND_CONFERENCE_MESSAGE :
				this.ipsObject.sendConferenceMessage( (String)messageObjects[0], (Message)messageObjects[1]);
				break;			
			case ADD_TO_BUDDY_LIST :
				this.ipsObject.addToBuddyList( (String)messageObjects[0], (String)messageObjects[1]);
				break;			
			case DELETE_FROM_BUDDY_LIST :
				this.ipsObject.deleteFromBuddyList( (String)messageObjects[0]);
				break;			
			case SEND_INSTANT_MESSAGE :
				this.ipsObject.sendInstantMessage( (String)messageObjects[0], (Message)messageObjects[1] );
				break;	
			case ADD_BUDDY_RESPONSE :
				this.ipsObject.addBuddyResponse((String)messageObjects[0], (Response)messageObjects[1]);
				break;
			case CONFERENCE_INVITATION_RESPONSE :
				this.ipsObject.conferenceInvitationResponse((String)messageObjects[0], (Response)messageObjects[1]);
				break;
			default:
				System.out.println("Recieved unknown message type:" + messageType);
				break;
		}
		} catch (EOFException e) {
			returnCode = false;
	    } catch (IOException e) {
	    	e.printStackTrace();
	    } catch(ClassNotFoundException e) {
	    	e.printStackTrace();
		} catch (IllegalStateException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return returnCode;
	}
	
    /** 
     * I run server side. 
     *	RCS Implement sendMessageToClient  This function should be called whenever the client needs to service a callback function.
     * @deprecated  Pass in an Object[] instead.
     */
	public void sendMessageToClient(Serializable origin, int type, Serializable payload) {
		System.out.println("Sending message " + type + " to client.");
		if(type <= END_OF_LISTENER_MESSAGE_TYPE_LIST) {
			throw new IllegalStateException("Invalid message type");
		}
		try { 
			sendMessage(origin, type, payload);
		} catch (IOException e) {
			e.printStackTrace();
			try {	// tear down the connections.
				in.close();
			} catch (IOException ioe) {
				ioe.printStackTrace();
			}
			try {
				out.close();
			} catch (IOException ioe) {
				ioe.printStackTrace();
			}
			System.exit(-1); // fail miserably
		}
	} 
	public boolean sendMessageToClient(int type, Serializable[] args) { 
		System.out.println("Sending message " + type + " to client.");
		if(type <= END_OF_LISTENER_MESSAGE_TYPE_LIST) {
			throw new IllegalStateException("Invalid message type");
		}
		try { 
			sendMessage(type, args);
		} catch (IOException e) { 
			e.printStackTrace();
			System.out.println(username + "'s connection died.  Un-registering it");
			InsecureServerLogic.insecureProtocolServers.remove(username);
			try {
			in.close();
			} catch (Exception e2) { 
				e.printStackTrace();
			}
			try { 
				out.close();
			} catch (Exception e2) { 
				e.printStackTrace();
			}
			//TODO error handling...
			return false;
		}
		return true;
	}

	/**
	 * I run client side.
	 * TODO RCS See sendMessageToClient.
	 */ 

	public boolean receiveMessageFromServer(IMListener listener, Protocol p) { 
		//receiveMessage();
		boolean returnCode = true;  // Don't disconnect by default.
		try {
			System.out.println("reading message from server.");
			int messageType = in.readInt();
			Object[] messageObjects = (Object[]) in.readObject();
			
			System.out.println("Received message from server");
			
			if(messageType <= END_OF_LISTENER_MESSAGE_TYPE_LIST) { 
				throw new IllegalArgumentException("Client received unknown message type: " + messageType);
			}
	
			switch (messageType) { 
				case CONNECTING:
				{
					listener.connecting(p);
					break; 
				}
				case CONNECTED:
				{
					listener.connected(p);
					break;
				}
				case CONNECT_FAILED:
				{
					listener.connectFailed(p, (String)messageObjects[0]);
					returnCode = false;
					break;
				}
				case DISCONNECTED:
				{
					listener.disconnected(p);
					returnCode = false;
					break;
				}
				case BUDDY_ADD_REQUEST:
				{
					Response r = listener.buddyAddRequest(new Buddy(p, (String)messageObjects[0]),
											 new Buddy(p, (String)messageObjects[1]),
											 (String)messageObjects[2]);
					sendMessageToServer(ADD_BUDDY_RESPONSE, new Serializable[] { (Serializable)messageObjects[0], r });
					break;
				}
				case BUDDY_ADDED:
				{
					listener.buddyAdded(new Buddy(p, (String)messageObjects[0]));
					break;
				}
				case BUDDY_ADD_REJECTED:
				{
					listener.buddyAddRejected(new Buddy(p, (String)messageObjects[0]), (String)messageObjects[1]);
					break;
				}
				case BUDDY_ADD_FAILED: 
				{
					listener.buddyAddFailed(new Buddy(p, (String)messageObjects[0]), (String)messageObjects[1]);
					break;
				}
				case BUDDY_DELETED:
				{
					listener.buddyDeleted(new Buddy(p, (String)messageObjects[0]));
					break;
				}
				case BUDDY_DELETE_FAILED:
				{         
					listener.buddyDeleteFailed(new Buddy(p, (String)messageObjects[0]), (String)messageObjects[1]);
					break;
				}
				case BUDDY_IGNORED:
				{
					listener.buddyIgnored(new Buddy(p, (String)messageObjects[0]));
					break;
				}
				case BUDDY_IGNORE_FAILED:
				{
					listener.buddyIgnoreFailed(new Buddy(p, (String)messageObjects[0]), (String)messageObjects[1]);
					break;
				}
				case BUDDY_UNIGNORED:
				{
					listener.buddyUnignored(new Buddy(p, (String)messageObjects[0]));
					break;
				}
				case BUDDY_UNIGNORE_FAILED:
				{
					listener.buddyUnignoreFailed(new Buddy(p, (String)messageObjects[0]), (String)messageObjects[1]);
					break;
				}
				case BUDDY_LIST_RECEIVED:
				{
					String[] buddies = (String[])(messageObjects[0]);
					Buddy[] buddyList = new Buddy[buddies.length];
					for (int i = 0; i < buddies.length; i++) {
						buddyList[i] = new Buddy(InsecureProtocol.instance, buddies[i]);
					}
					listener.buddyListReceived(p, buddyList);
					break;
				}
				case IGNORE_LIST_RECEIVED:
				{
					String[] ignored = (String[])(messageObjects[0]);
					Buddy[] ignoreList = new Buddy[ignored.length];
					for(int i = 0; i< ignored.length; i++) {
						ignoreList[i] = new Buddy(InsecureProtocol.instance, ignored[i]);
					}
					listener.ignoreListReceived(p, ignoreList);
					 break;
				}
				case BUDDY_STATUS_CHANGED:
				{					Buddy b = new Buddy(p, (String)(messageObjects[0]));
					b.setStatus((String)(messageObjects[1]));
					listener.buddyStatusChanged(b);
					break;
				}
				case INSTANT_MESSAGE_RECEIVED:
				{
					Buddy buddy = new Buddy(InsecureProtocol.instance, (String)(messageObjects[0]));
					listener.instantMessageReceived(buddy, (Message)(messageObjects[1]));
					break;
				}
				case OFFLINE_MESSAGE_RECEIVED:
				{
					break;
				}
				case PROTOCOL_MESAGE_RECEIVED:
				{
					break;
				}
				case TYPING_STARTED:
				{
					break;
				}
				case TYPING_STOPPED:
				{
					break;
				}
				case CONFERENCE_INVITATION_RECEIVED:
				{
					String conferenceName = (String)messageObjects[0];
					String [] inviteeNames = (String[]) messageObjects[3];
					Buddy[] invitees = new Buddy[inviteeNames.length];
					for(int i = 0; i < invitees.length; i++) {
						invitees[i] = new Buddy(this.protocol, inviteeNames[i]);
					}
					Conference conference = new Conference(InsecureProtocol.instance, 
							  new Buddy(InsecureProtocol.instance, 
									  	(String)messageObjects[1]), 
							  invitees);
					protocol.conferences.put(conferenceName, conference);
					Response response = listener.conferenceInvitationReceived(conference,  
														  (String)messageObjects[2]);
					sendMessageToServer(InsecureNetworking.CONFERENCE_INVITATION_RESPONSE, new Serializable[] { conferenceName, response });
					break;
				}
				case CONFERENCE_INVITATION_DECLINED:
					listener.conferenceInvitationDeclined((Conference)protocol.conferences.get(messageObjects[0]), 
														  new Buddy(this.protocol, (String)messageObjects[1]),
														  (String)messageObjects[2]);
					break;
				case CONFERENCE_INVITATION_ACCEPTED:
				{
					listener.conferenceInvitationAccepted((Conference)protocol.conferences.get(messageObjects[0]),
														   new Buddy(this.protocol, (String)messageObjects[1]));
					break;
				}
				case CONFERENCE_MESSAGE_RECEIVED:
				{
					listener.conferenceMessageReceived((Conference)protocol.conferences.get(messageObjects[0]),
														new Buddy(this.protocol, (String)messageObjects[1]),
														(Message)messageObjects[2]);
					break;
					
				}
				case CONFERENCE_PARTICIPANT_JOINED:
				{
					listener.conferenceParticipantJoined((Conference)protocol.conferences.get(messageObjects[0]), 
														 new Buddy(this.protocol, (String)messageObjects[1]));
					break;
				}
				case CONFERENCE_PARTICIPANT_LEFT:
				{
					listener.conferenceParticipantLeft((Conference)protocol.conferences.get(messageObjects[0]), 
							 							new Buddy(this.protocol, (String)messageObjects[1]));
					break;
				}
				case CONFERENCE_CLOSED:
					listener.conferenceClosed((Conference)protocol.conferences.get(messageObjects[0]));
					break;
				case MAIL_NOTIFICATION_RECEIVED:
					break;
				default:
					System.out.println("Recieved unknown message type:" + messageType);
					break;
			}
		} catch (Exception e) {
			// TODO lame error handling.
			e.printStackTrace();
			return false;
		}
		return returnCode;
	} 
}
